package digi.recipeManager;

import java.util.List;

import org.bukkit.command.*;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;

import digi.recipeManager.data.*;

class CmdRecipes implements CommandExecutor
{
	@Override
	public boolean onCommand(final CommandSender sender, final Command command, final String label, final String[] args)
	{
		if(!Main.allowedToCraft(sender))
			return true;
		
		if(args.length <= 0)
		{
			Main.printMessage(sender, Lang.get("command.rmrecipes.usage", "recipes"));
			Main.printMessage(sender, Lang.get("command.rmrecipes.workbenchrecipes", Recipes.craft.size(), Recipes.combine.size()));
			Main.printMessage(sender, Lang.get("command.rmrecipes.furnacerecipes", Recipes.smelt.size(), Recipes.fuels.size()));
			
			return true;
		}
		
		final int perPage = 10;
		final String itemStr = args[0].toUpperCase();
		int page = 0;
		Item item;
		boolean ingredient;
		List<Recipe> recipes;
		int recipesNum;
		
		String name = null;
		
		if(sender instanceof Player)
			name = ((Player)sender).getName();
		
		final boolean next = itemStr.equals("NEXT");
		final boolean prev = itemStr.equals("PREV");
		
		if(next || prev) // next page
		{
			final Page data = Recipes.playerPage.get(name);
			
			if(data == null)
				return true;
			
			item = data.item;
			ingredient = data.ingredient;
			recipes = Recipes.getRecipesForItem(item, ingredient);
			recipesNum = recipes.size();
			
			if((prev && (data.page - 1) < 0) || (next && ((data.page + 1) * perPage) > recipesNum))
				return true;
			
			if(next)
				data.page++;
			else
				data.page--;
			
			page = data.page;
			
			Recipes.playerPage.put(name, data);
		}
		else
		{
			if(itemStr.equals("THIS"))
			{
				if(!(sender instanceof Player))
				{
					sender.sendMessage("Only players can use the 'THIS' argument.");
					return true;
				}
				
				final ItemStack itemStack = ((Player)sender).getItemInHand();
				
				if(itemStack == null || itemStack.getTypeId() == 0)
				{
					Main.printMessage(sender, Lang.get("command.rmrecipes.invalidhelditem"));
					return true;
				}
				else
				{
					item = new Item(itemStack);
					item.amount = 0;
					item.enchantments.clear();
				}
			}
			else
			{
				item = Main.processItem(itemStr, 0, true, false, false, false);
				
				if(item == null)
				{
					Main.printMessage(sender, Lang.get("command.rmrecipes.invaliditem", itemStr.toLowerCase()));
					return true;
				}
			}
			
			ingredient = (args.length > 1);
			recipes = Recipes.getRecipesForItem(item, ingredient);
			recipesNum = recipes.size();
			
			if(recipesNum == 0)
			{
				Main.printMessage(sender, Lang.get((ingredient ? "command.rmrecipes.noingredient" : "command.rmrecipes.noresult"), item.printAuto()));
				return true;
			}
			
			page = 0;
			Recipes.playerPage.put(name, new Page(item, ingredient, page));
		}
		
		final int maxPages = (int)Math.ceil(recipesNum / (0.0 + perPage));
		
		sender.sendMessage(" ");
		Main.printMessage(sender, Lang.get((ingredient ? "command.rmrecipes.listingredient" : "command.rmrecipes.listresult"), item, recipesNum, Lang.get("general.recipe" + (recipesNum == 1 ? "" : "s"))));
		
		if(recipesNum > perPage)
			Main.printMessage(sender, Lang.get("command.rmrecipes.pageofpages", (page + 1), maxPages));
		
		sender.sendMessage(" ");
		
		Recipe r;
		final int start = (page * perPage);
		final int end = (start + perPage);
		
		for(int i = start; i < end; i++)
		{
			if(i >= recipesNum)
				break;
			
			r = recipes.get(i);
			
			if(r.permission != null && !sender.hasPermission(r.permission))
				continue;
			
			if(sender instanceof Player && !r.usableWorld(((Player)sender).getWorld().getName()))
				continue;
			
			Main.printMessage(sender, r.print());
			
			// TODO: finish these
			
			/*
			if(r.minExp > 0 || r.maxExp > 0 || r.minLevel > 0 || r.maxLevel > 0 || r.minMoney > 0 || r.maxMoney > 0)
			{
				Main.printMessage(sender, "Requirements: ???");
			}
			*/
			
			/*
			StringBuffer rewards = new StringBuffer();
			
			if(r.giveExp != 0)
				rewards.append(" " + (r.giveExp > 0 ? "+" : "-") + Math.abs(r.giveExp) + " experience");
			
			if(r.giveLevel != 0)
				rewards.append(" " + (r.giveLevel > 0 ? "+" : "-") + Math.abs(r.giveLevel) + " level");
			
			if(Econ.enabled)
			{
				if(r.giveMoney != 0)
					rewards.append(" " + (r.giveMoney > 0 ? "+" : "-") + Econ.moneyFormat(Math.abs(r.giveMoney)));
			}
			
			if(rewards.length() > 0)
				Main.printMessage(sender, "Rewards and costs: " + rewards);
			
			if(sender.isOp())
			{
				if(r.permission != null)
					sender.sendMessage("Permission: " + r.permission);
				
				if(r.worlds.size() > 0)
				{
					StringBuffer worlds = new StringBuffer("Worlds: ");
					
					for(String worldName : r.worlds)
					{
						worlds.append(worldName + ", ");
					}
					
					sender.sendMessage(worlds.substring(worlds.length() - 2, worlds.length()));
				}
			}
			*/
			
			sender.sendMessage(" ");
		}
		
		if(recipesNum > end)
		{
			Main.printMessage(sender, Lang.get("command.rmrecipes.nextavailable", (page + 1), maxPages, "recipes next"));
			return true;
		}
		else if(page > 0)
		{
			Main.printMessage(sender, Lang.get("command.rmrecipes.prevavailable", (page + 1), maxPages, "recipes prev"));
		}
		
		sender.sendMessage(" ");
		
		return true;
	}
}

class CmdCheck implements CommandExecutor
{
	@Override
	public boolean onCommand(final CommandSender sender, final Command command, final String label, final String[] args)
	{
		Main.printMessage(sender, Lang.get("command.rmcheck.checking", "recipes"));
		
		if(Main.loadRecipes(true))
			Main.printMessage(sender, Lang.get("command.rmcheck.valid"));
		else if(sender instanceof Player)
			Main.printMessage(sender, Lang.get("command.rmcheck.errors"));
		
		return true;
	}
}

class CmdReload implements CommandExecutor
{
	@Override
	public boolean onCommand(final CommandSender sender, final Command command, final String label, final String[] args)
	{
		Main.printMessage(sender, Lang.get("command.rmreload.reloading"));
		
		final char lastExistingRecipes = Config.ExistingRecipes.character;
		
		Main.loadSettings();
		
		if(Config.ExistingRecipes.character == 'n' && lastExistingRecipes != Config.ExistingRecipes.character)
		{
			Main.log.info("The 'existing-recipes' was changed to 'nothing', plugin will attempt to restore all recipes but it can't restore recipes from other plugins or mods.");
			Main.sv.resetRecipes();
		}
		
		if(Main.loadRecipes(false))
			Main.printMessage(sender, Lang.get("command.rmreload.done"));
		else if(sender instanceof Player)
			Main.printMessage(sender, Lang.get("command.rmreload.errors"));
		
		return true;
	}
}