package digi.recipeManager;

import java.util.HashMap;

import org.bukkit.*;
import org.bukkit.block.*;
import org.bukkit.entity.Player;
import org.bukkit.event.Event.Result;
import org.bukkit.event.*;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.inventory.*;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.event.world.*;
import org.bukkit.inventory.*;
import org.bukkit.inventory.Recipe;

import digi.recipeManager.data.*;

class Events implements Listener
{
	protected Events()
	{
		Main.pm.registerEvents(this, Main.plugin);
		
		for(final World world : Main.sv.getWorlds())
		{
			worldLoad(world);
		}
	}
	
	@EventHandler
	public void playerQuit(final PlayerQuitEvent event)
	{
		Recipes.playerPage.remove(event.getPlayer().getName());
	}
	
	@EventHandler(priority = EventPriority.LOW, ignoreCancelled = true)
	public void eventInventoryClick(final InventoryClickEvent event)
	{
		final Inventory inv = event.getInventory();
		
		if(inv == null || !(inv instanceof FurnaceInventory))
			return;
		
		final InventoryHolder holder = inv.getHolder();
		
		if(holder == null || !(holder instanceof Furnace))
			return;
		
		final Player player = (Player)event.getWhoClicked();
		final ItemStack cursorItem = event.getCursor();
		
		if(cursorItem == null)
			return;
		
		switch(event.getRawSlot())
		{
			case 0: // INGREDIENT slot
			{
				if(!Main.allowedToCraft(player))
				{
					event.setCancelled(true);
					event.setResult(Result.DENY);
					return;
				}
				
				final Smelt recipe = Recipes.getSmeltRecipe(cursorItem);
				
				if(recipe == null)
					return;
				
				if(!recipe.usableBy(player, true))
				{
					event.setCancelled(true);
					event.setResult(Result.DENY);
				}
				
				return;
			}
			
			case 1: // FUEL slot
			{
				if(!Main.allowedToCraft(player))
				{
					event.setCancelled(true);
					event.setResult(Result.DENY);
					return;
				}
				
				final Fuel recipe = Recipes.getFuelRecipe(cursorItem);
				
				if(recipe == null)
					return;
				
				if(!recipe.usableBy(player, true))
				{
					event.setCancelled(true);
					event.setResult(Result.DENY);
				}
				
				return;
			}
			
			case 3:
				return; // Result slot
				
			default: // player inventory
			{
				if(event.isShiftClick())
				{
					event.setCancelled(true);
					return;
				}
			}
		}
	}
	
	@EventHandler(priority = EventPriority.LOW, ignoreCancelled = true)
	public void eventPreCraft(final PrepareItemCraftEvent event)
	{
		final Player player = (Player)event.getView().getPlayer();
		
		if(!Main.allowedToCraft(player))
		{
			event.getInventory().setItem(0, null);
			return;
		}
		
		if(event.isRepair())
		{
			if(!Config.RepairRecipes.bool)
			{
				event.getInventory().setItem(0, null);
				
				if(Config.PrintNoRepair.bool)
					Main.printMessage(player, Lang.get("notify.norepair"));
			}
			
			return;
		}
		
		final Recipe craftedRecipe = event.getRecipe();
		
		if(craftedRecipe == null)
			return;
		
		final ItemStack craftedResult = craftedRecipe.getResult();
		
		if(!Recipes.isCustomRecipe(craftedResult))
			return;
		
		if(craftedRecipe instanceof ShapedRecipe)
		{
			final Craft recipe = Recipes.craft.get(craftedResult.getAmount());
			
			if(recipe == null)
				return;
			
			ItemStack result = null;
			
			if(recipe.usableBy(player, true))
				result = recipe.results.get(0).getItemStack(); // Display the first result item
				
			event.getInventory().setItem(0, result);
		}
		else if(craftedRecipe instanceof ShapelessRecipe)
		{
			final Combine recipe = Recipes.combine.get(craftedResult.getAmount());
			
			if(recipe == null)
				return;
			
			ItemStack result = null;
			
			if(recipe.usableBy(player, true))
				result = recipe.results.get(0).getItemStack(); // Display the first result item
				
			event.getInventory().setItem(0, result);
		}
	}
	
	@SuppressWarnings("deprecation")
	@EventHandler(priority = EventPriority.LOW, ignoreCancelled = true)
	public void eventCraft(final CraftItemEvent event)
	{
		if(event.getCurrentItem() == null || event.getCurrentItem().getType() == Material.AIR)
			return;
		
		final Recipe r = event.getRecipe();
		
		if(r == null)
			return;
		
		final ItemStack recipeResult = r.getResult();
		
		if(!Recipes.isCustomRecipe(recipeResult))
			return;
		
		final Player player = (Player)event.getWhoClicked();
		
		if(r instanceof ShapedRecipe)
		{
			final Craft recipe = Recipes.craft.get(recipeResult.getAmount());
			
			if(recipe == null)
				return;
			
			if(event.isShiftClick() && (recipe.giveExp != 0 || recipe.giveLevel != 0 || (Econ.enabled && recipe.giveMoney != 0)))
			{
				Main.printMessage(player, Lang.get("noshiftclick.rewards"));
				event.setCancelled(true);
				return;
			}
			
			recipe.affectExp(player);
			recipe.affectLevel(player);
			recipe.affectMoney(player);
			
			if(recipe.results.size() > 1)
			{
				if(event.isShiftClick())
				{
					Main.printMessage(player, Lang.get("noshiftclick.multipleresults"));
					event.setCancelled(true);
					return;
				}
				
				final Item result = recipe.getRandomResult();
				final ItemStack cursor = event.getCursor();
				
				if(result.type == 0)
				{
					if(Config.PrintFailure.bool)
						Main.printMessage(player, Lang.get("notify.failure", result.chance + "%"));
					
					final CraftingInventory inv = event.getInventory();
					ItemStack item;
					int amt;
					
					for(int i = 1; i < 10; i++)
					{
						item = inv.getItem(i);
						
						if(item != null)
						{
							if((amt = (item.getAmount() - 1)) > 0)
								item.setAmount(amt);
							else
								inv.clear(i);
						}
					}
					
					event.setResult(Result.DENY);
					event.setCancelled(true);
				}
				else
				{
					if(cursor.getTypeId() != 0 && !result.compareItemStack(cursor))
					{
						final HashMap<Integer, ItemStack> extra = player.getInventory().addItem(cursor);
						player.updateInventory();
						
						if(extra.size() > 0)
						{
							for(final ItemStack i : extra.values())
								player.getWorld().dropItem(player.getLocation(), i);
						}
						
						event.setCursor(result.getItemStack());
					}
					else
						event.setCurrentItem(result.getItemStack());
				}
				
				return;
			}
		}
		else if(r instanceof ShapelessRecipe)
		{
			final Craft recipe = Recipes.craft.get(recipeResult.getAmount());
			
			if(recipe == null)
				return;
			
			if(event.isShiftClick() && (recipe.giveExp != 0 || recipe.giveLevel != 0 || (Econ.enabled && recipe.giveMoney != 0)))
			{
				player.sendMessage("Can't use Chift+Click on recipes that give rewards for now, sorry.");
				return;
			}
			
			recipe.affectExp(player);
			recipe.affectLevel(player);
			recipe.affectMoney(player);
			
			if(recipe.results.size() > 1)
			{
				if(event.isShiftClick())
				{
					Main.printMessage(player, Lang.get("noshiftclick.multipleresults"));
					event.setCancelled(true);
					return;
				}
				
				final Item result = recipe.getRandomResult();
				final ItemStack cursor = event.getCursor();
				
				if(result.type == 0)
				{
					if(Config.PrintFailure.bool)
						Main.printMessage(player, Lang.get("notify.failure", result.chance + "%"));
					
					final CraftingInventory inv = event.getInventory();
					ItemStack item;
					int amt;
					
					for(int i = 1; i < 10; i++)
					{
						item = inv.getItem(i);
						
						if(item != null)
						{
							if((amt = (item.getAmount() - 1)) > 0)
								item.setAmount(amt);
							else
								inv.clear(i);
						}
					}
					
					event.setResult(Result.DENY);
					event.setCancelled(true);
				}
				else
				{
					if(cursor.getTypeId() != 0 && !result.compareItemStack(cursor))
					{
						final HashMap<Integer, ItemStack> extra = player.getInventory().addItem(cursor);
						player.updateInventory();
						
						if(extra.size() > 0)
						{
							for(final ItemStack i : extra.values())
								player.getWorld().dropItem(player.getLocation(), i);
						}
						
						event.setCursor(result.getItemStack());
					}
					else
						event.setCurrentItem(result.getItemStack());
				}
				
				return;
			}
		}
		
		Main.bs.scheduleSyncDelayedTask(Main.plugin, new Runnable() // temp bugfix for Bukkit and for returned items
		{
			@Override
			public void run()
			{
				player.updateInventory();
			}
		});
	}
	
	@EventHandler(priority = EventPriority.LOW, ignoreCancelled = true)
	public void eventFurnaceSmelt(final FurnaceSmeltEvent event)
	{
		final Smelt recipe = Recipes.getSmeltRecipe(event.getSource());
		
		if(recipe == null)
			return;
		
		if(recipe.result.type == 0 || !recipe.getChanceResult())
		{
			final BlockState blockState = event.getBlock().getState();
			
			if(!(blockState instanceof Furnace))
				return;
			
			event.setCancelled(true); // setting the result to null will make the client timeout and the server spit errors
			final Furnace furnace = (Furnace)blockState;
			final ItemStack item = furnace.getInventory().getSmelting();
			final int amount = item.getAmount() - 1;
			
			if(amount <= 0)
				furnace.getInventory().clear(0);
			else
				item.setAmount(amount);
		}
	}
	
	@EventHandler(ignoreCancelled = true)
	public void eventFurnaceBurn(final FurnaceBurnEvent event)
	{
		final Block block = event.getBlock();
		final Location loc = block.getLocation();
		
		if(!Main.furnacesSmelting.containsKey(loc))
			Main.furnacesSmelting.put(loc, (double)0);
		
		final ItemStack ingredient = event.getFuel();
		
		if(Config.FuelReturnBuckets.bool)
		{
			switch(ingredient.getType())
			{
				case WATER_BUCKET:
				case LAVA_BUCKET:
				case MILK_BUCKET:
				{
					Main.bs.scheduleSyncDelayedTask(Main.plugin, new Runnable()
					{
						@Override
						public void run()
						{
							final BlockState blockState = event.getBlock().getState();
							
							if(blockState instanceof Furnace)
								((Furnace)blockState).getInventory().setItem(1, new ItemStack(Material.BUCKET, 1));
						}
					});
					
					break;
				}
			}
		}
		
		final Fuel fuel = Recipes.getFuelRecipe(ingredient);
		
		if(fuel == null)
			return;
		
		event.setBurnTime(20 * fuel.getTime());
		event.setBurning(true);
	}
	
	// Monitor furnaces
	
	@EventHandler(priority = EventPriority.MONITOR, ignoreCancelled = true)
	public void eventBlockBreak(final BlockBreakEvent event)
	{
		if(event.getBlock().getType() == Material.BURNING_FURNACE)
			Main.furnacesSmelting.remove(event.getBlock().getLocation());
	}
	
	@EventHandler(priority = EventPriority.MONITOR, ignoreCancelled = true)
	public void eventWorldLoad(final WorldLoadEvent event)
	{
		worldLoad(event.getWorld());
	}
	
	@EventHandler(priority = EventPriority.MONITOR, ignoreCancelled = true)
	public void eventChunkLoad(final ChunkLoadEvent event)
	{
		if(!event.isNewChunk())
			furnaceChunk(event.getChunk(), true);
	}
	
	@EventHandler(priority = EventPriority.MONITOR, ignoreCancelled = true)
	public void eventChunkUnload(final ChunkUnloadEvent event)
	{
		furnaceChunk(event.getChunk(), false);
	}
	
	private void worldLoad(final World world)
	{
		final Chunk chunks[] = world.getLoadedChunks();
		
		for(final Chunk chunk : chunks)
		{
			furnaceChunk(chunk, true);
		}
	}
	
	private void furnaceChunk(final Chunk chunk, final boolean add)
	{
		if(chunk == null)
			return;
		
		final BlockState tileEnts[] = chunk.getTileEntities();
		
		if(tileEnts.length <= 0)
			return;
		
		for(final BlockState ent : tileEnts)
		{
			if(ent != null && ent instanceof Furnace && ent.getType() == Material.BURNING_FURNACE)
			{
				if(add)
					Main.furnacesSmelting.put(ent.getBlock().getLocation(), (double)0);
				else
					Main.furnacesSmelting.remove(ent.getBlock().getLocation());
			}
		}
	}
}