package digi.recipeManager;

import java.io.File;

import org.bukkit.configuration.file.*;

public class Lang
{
	private static FileConfiguration langCfg = null;
	
	public static void load()
	{
		if(Config.Language.string.equals("en"))
		{
			defaultLang();
			return;
		}
		
		final File file = new File(Main.pluginFolder + File.separator + "lang_" + Config.Language.string + ".yml");
		
		if(!file.exists())
		{
			Main.log.warning("lang_" + Config.Language.string + ".yml is missing! Using lang_en.yml.");
			defaultLang();
			return;
		}
		
		langCfg = YamlConfiguration.loadConfiguration(file);
		
		if(!validVersion())
		{
			Main.log.warning("lang_" + Config.Language.string + ".yml is an older version, update it! Using lang_en.yml.");
			defaultLang();
			return;
		}
		
		Main.log.fine("lang_" + Config.Language.string + ".yml loaded.");
	}
	
	private static boolean defaultLang()
	{
		final File file = new File(Main.pluginFolder + File.separator + "lang_en.yml");
		
		if(!file.exists()) // in case it's missing
			Main.plugin.saveResource("lang_en.yml", false);
		
		langCfg = YamlConfiguration.loadConfiguration(file);
		
		if(!validVersion()) // in case it's not missing but it's old
		{
			Main.plugin.saveResource("lang_en.yml", true);
			
			langCfg = YamlConfiguration.loadConfiguration(file);
			
			Main.log.fine("lang_en.yml was overwritten with the newer version.");
		}
		
		return true;
	}
	
	private static boolean validVersion()
	{
		try
		{
			final String version = langCfg.getString("rm", null);
			
			return (version != null && Main.pluginVersion.equals(version));
		}
		catch(final Exception e)
		{
			return false;
		}
	}
	
	public static String get(final String key, final Object... args)
	{
		final String str = langCfg.getString(key);
		
		if(str == null)
		{
			Main.log.severe("lang_" + Config.Language.string + ".yml is missing key '" + key + "' !");
			return "(language key missing: '" + key + "')";
		}
		
		return (args.length > 0 ? String.format(str, args) : str);
	}
}
